from abc import ABC, abstractclassmethod
import pygame
from threading import Thread
from time import time
from pygame import surface
import numpy as np



class PygameVisulizer():
    width = 1000
    height = 1000
    running = True
    functions = []
    efunctions = []
    fps_value_queue = []
    dt = 0

    def __init__(self, width, height) -> None:
        self.width, self.height = width, height
        self.surface = pygame.display.set_mode((self.width, self.height))

    def register_draw_function(self, f):
        """
        f: function, f(pygame.Surface) -> None
        """
        self.functions.append(f)

    def register_event_function(self, f):
        self.efunctions.append(f)

    def avg_fps(self):
        return np.average(self.fps_value_queue)

    def run(self):
        pygame.font.init()
        sysfont = pygame.font.SysFont(pygame.font.get_default_font(), 30)
        while self.running:
            loop_start = time()

            # event loop
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.running = False
                for f in self.efunctions:
                    f(event)
                
            # draw loop
            for f in self.functions:
                f(self.surface)


            # update average fps
            self.dt = time() - loop_start
            self.fps_value_queue.insert(0, 1 / self.dt if self.dt != 0 else 1)
            while len(self.fps_value_queue) > 100:
                self.fps_value_queue.pop(-1)
            
            # draw fps
            text = sysfont.render(f"{round(self.avg_fps(), 2):3.3} fps", True, (255, 255, 255), 0)
            self.surface.blit(text, text.get_rect().topleft)
            
            # update screen
            pygame.display.flip()
            self.surface.fill(0)